#ifndef PPM_EXTENTS_H
#define PPM_EXTENTS_H
/**
 * @file ppm_extents.h
 * @brief declarations for functions on extents
 *
 * @copyright Copyright  (C)  2011  Thomas Jahns <jahns@dkrz.de>
 *
 * @version 1.0
 * @author Thomas Jahns <jahns@dkrz.de>
 */
/*
 * Maintainer: Thomas Jahns <jahns@dkrz.de>
 * URL: https://www.dkrz.de/redmine/projects/show/scales-ppm
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 * integer range description
 */
struct extent
{
  int first, /**< the range is anchored at \a first */
    size;  /**< the range has size \a size, i.e. the range equals
            * \li [first, first + size - 1] for size > 0
            * \li [first + size + 1, first] for size < 0
            * \li the empty set if size = 0
            */
};

/**
 * inclusive integer range description
 *
 * describes range [first,last]
 */
struct iinterval
{
  int first,   /**< the range is anchored at \a first */
    last;      /**< the range contains \a last */
};

/**
 * inclusive float range description
 *
 * describes range [first,last]
 */
struct iinterval_sp
{
  float first,  /**< the range is anchored at \a first */
    last;       /**< the range contains \a last */
};

/**
 * inclusive double range description
 *
 * describes range [first,last]
 */
struct iinterval_dp
{
  double first, /**< the range is anchored at \a first */
    last;       /**< the range contains \a last */
};

/**
 * Write string representation of \a extent to \a buf.
 * @param buf must point to array at least 26 char long.
 * @param ext range to print
 */
void
PPM_sprint_extent(char buf[], struct extent *ext);

/*
 * Local Variables:
 * license-project-url: "https://www.dkrz.de/redmine/projects/show/scales-ppm"
 * license-default: "bsd"
 * End:
 */
#endif
